# Problem with include paths

## Content

This "minimial-(non)working-example" (MWE) contains 

- `Foo.C` - a simple class to compile and load into ROOT 
- `FooBuild.C` - a script to compile the above 

## The problem 

If the user specifies and include path like 

	-I /some/path/with-xin-it 
	
f.ex. via calling `gSystem->AddIncludePath`, then the compilation
fails. 

Note the space between the `-I` and the path.  Also note the path
contain the substring `-x`.   

What happens in this case is that ROOT tries to split the command line
arguments for the internal call to the compiler on what it believes to
be option boundaries.   This means that the string above is split as
two separate arguments: 

    -I /some/path/wit
	-xin-it 
	
If we quote the include path string as 

    -I" /some/path/with-xin-it"
	
then the option is maintained in full (though the current directory is
prefixed to the path).

## To reproduce 

To see the problem do 

	root FooBuild.C 
	
To see the situation where the compilation works, do 

	root FooBuild.C\(0\) 
	
## Details 

The output in case of failure (anonymous paths)

    Processing FooBuild.C...
	Info in <TUnixSystem::ACLiC>: creating shared library /pwd/./Foo_C.so
	Info in <ACLiC>: echo > "/pwd/Foo_C.d"
	Info in <ACLiC>: /inst/bin/rmkdepend \
		"-f/pwd/Foo_C.d" \
		-o_C.so  \
		-Y \
		--  \
		"-I/inst/include"  \
		-I/inst/include \
		-I"/pwd/ /some/path/wit" \
		-xin-it \
		-I"/inst/etc" \
		-I"/inst/etc/cling" \
		-I"/inst/etc/cling/plugins/include" \
		-I"/inst/include" \
		-I"/build/root/include" \
		-I"/build/root/include/"    \
		-- "/pwd/./Foo.C"  "-t./Foo_C.so" > /dev/null 2>&1 
	Info in <ACLiC>: echo ./Foo_C.so: \
		/inst/include/RVersion.h \
		/inst/include/RConfig.h \
		/inst/include/TClass.h \
		/inst/include/TDictAttributeMap.h \
		/inst/include/TInterpreter.h \
		/inst/include/TROOT.h \
		/inst/include/TBuffer.h \
		/inst/include/TMemberInspector.h \
		/inst/include/TError.h \
		/inst/include/RtypesImp.h \
		/inst/include/TIsAProxy.h \
		/inst/include/TFileMergeInfo.h \
		/inst/include/TCollectionProxyInfo.h \
		/inst/bin/rootcling  \
		>> "/pwd/Foo_C.d"
	Info in <ACLiC>: creating the dictionary files
	Info in <ACLiC>: /inst/bin/rootcling -v0 \
		"--lib-list-prefix=/pwd/Foo_C_ACLiC_map" \
		-f "/pwd/Foo_C_ACLiC_dict.cxx" \
		-I/inst/include \
		-I /some/path/with-xin-it \
		-I"/inst/etc" \
		-I"/inst/etc/cling" \
		-I"/inst/etc/cling/plugins/include" \
		-I"/inst/include" \
		-I"/build/root/include" \
		-I"/build/root/include/" \
		-D__ACLIC__  "/pwd/./Foo.C" "/pwd/Foo_C_ACLiC_linkdef.h"
	Info in <TUnixSystem::Load>: loaded library \
		/inst/lib/libCore.so.6.26.00, status 1
	Info in <ACLiC>: compiling the dictionary and script files
	Info in <ACLiC>: cd "/pwd" ; \
		c++ \
		-fPIC \
		-c \
		-g  \
		-std=c++17 \
		-Wno-implicit-fallthrough \
		-Wno-noexcept-type \
		-pipe \
		-W \
		-Woverloaded-virtual \
		-fsigned-char \
		-pthread  \
		-I/inst/include \
		-I"/pwd/ /some/path/wit" \
		-xin-it \
		-I"/inst/etc" -I"/inst/etc/cling" \
		-I"/inst/etc/cling/plugins/include" \
		-I"/inst/include" -I"/build/root/include" \
		-I"/build/root/include/"   \
		-D__ACLIC__ \
		"/pwd/Foo_C_ACLiC_dict.cxx" ; \
		c++ \
		-g  "/pwd/Foo_C_ACLiC_dict.o" \
		-shared   "/lib/x86_64-linux-gnu/libm.so" \
		"/lib/x86_64-linux-gnu/libpthread.so" \
		"/lib/x86_64-linux-gnu/libdl.so" \
		"/lib/x86_64-linux-gnu/libzstd.so" \
		"/lib/x86_64-linux-gnu/libz.so" \
		"/lib/x86_64-linux-gnu/liblz4.so" \
		"/lib/x86_64-linux-gnu/liblzma.so" \
		"/lib/x86_64-linux-gnu/libpcre.so" \
		"/lib/x86_64-linux-gnu/libc.so" \
		"/inst/lib/libRint.so.6.26" \
		"/inst/lib/libCore.so.6.26" \
		"/inst/lib/libRIO.so" \
		"/inst/lib/libThread.so.6.26" \
		"/inst/lib/libCling.so" \
		"/inst/lib/libMathCore.so" \
		"/inst/lib/libImt.so.6.26" \
		"/inst/lib/libMultiProc.so.6.26" \
		"/inst/lib/libNet.so.6.26" \
		-o "/pwd/./Foo_C.so"
	c++: error: language in-it not recognized
	c++: error: language in-it not recognized
	/usr/bin/ld: cannot find /pwd/Foo_C_ACLiC_dict.o: No such file or directory
	collect2: error: ld returned 1 exit status
	Error in <ACLiC>: Executing 'cd "/pwd" ; \
		c++ \
		-fPIC \
		-c \
		-g  \
		-std=c++17 \
		-Wno-implicit-fallthrough \
		-Wno-noexcept-type \
		-pipe \
		-W \
		-Woverloaded-virtual \
		-fsigned-char \
		-pthread  \
		-I/inst/include \
		-I"/pwd/ /some/path/wit" \
		-xin-it \
		-I"/inst/etc" -I"/inst/etc/cling" \
		-I"/inst/etc/cling/plugins/include" \
		-I"/inst/include" -I"/build/root/include" \
		-I"/build/root/include/"   \
		-D__ACLIC__ "/pwd/Foo_C_ACLiC_dict.cxx" ; \
		c++ -g  "/pwd/Foo_C_ACLiC_dict.o" \
		-shared   \
		"/lib/x86_64-linux-gnu/libm.so" \
		"/lib/x86_64-linux-gnu/libpthread.so" \
		"/lib/x86_64-linux-gnu/libdl.so" \
		"/lib/x86_64-linux-gnu/libzstd.so" \
		"/lib/x86_64-linux-gnu/libz.so" \
		"/lib/x86_64-linux-gnu/liblz4.so" \
		"/lib/x86_64-linux-gnu/liblzma.so" \
		"/lib/x86_64-linux-gnu/libpcre.so" \
		"/lib/x86_64-linux-gnu/libc.so" \
		"/inst/lib/libRint.so.6.26" \
		"/inst/lib/libCore.so.6.26" \
		"/inst/lib/libRIO.so" \
		"/inst/lib/libThread.so.6.26" \
		"/inst/lib/libCling.so" \
		"/inst/lib/libMathCore.so" \
		"/inst/lib/libImt.so.6.26" \
		"/inst/lib/libMultiProc.so.6.26" \
		"/inst/lib/libNet.so.6.26" -o "/pwd/./Foo_C.so"' failed!

Note 

		-I"/pwd/ /some/path/wit" \
		-xin-it \

above (Ignore prefixing the current directory for now).

The output in case of success  (anonymous paths)


	Processing FooBuild.C(0)...
	Info in <TUnixSystem::ACLiC>: creating shared library /pwd/./Foo_C.so
	Info in <ACLiC>: echo > "/pwd/Foo_C.d"
	Info in <ACLiC>: /inst/bin/rmkdepend \
		"-f/pwd/Foo_C.d" \
		-o_C.so  \
		-Y \
		--  \
        "-I/inst/include"  \
        -I/inst/include \
        -I"/pwd/ /some/path/with-xin-it" \
        -I"/inst/etc" \
        -I"/inst/etc/cling" \
        -I"/inst/etc/cling/plugins/include" \
        -I"/inst/include" \
        -I"/build/root/include" \
        -I"/build/root/include/" \
		-- \
		"/pwd/./Foo.C"  \
		"-t./Foo_C.so" > /dev/null 2>&1 
	Info in <ACLiC>: echo ./Foo_C.so: /inst/include/RVersion.h 
        /inst/include/RConfig.h 
        /inst/include/TClass.h 
        /inst/include/TDictAttributeMap.h 
        /inst/include/TInterpreter.h 
        /inst/include/TROOT.h 
        /inst/include/TBuffer.h 
        /inst/include/TMemberInspector.h 
        /inst/include/TError.h 
        /inst/include/RtypesImp.h 
        /inst/include/TIsAProxy.h 
        /inst/include/TFileMergeInfo.h 
        /inst/include/TCollectionProxyInfo.h /inst/bin/rootcling  \
		>> "/pwd/Foo_C.d"
	Info in <ACLiC>: creating the dictionary files
	Info in <ACLiC>: /inst/bin/rootcling -v0 \
        "--lib-list-prefix=/pwd/Foo_C_ACLiC_map" \
		-f "/pwd/Foo_C_ACLiC_dict.cxx" \
        -I/inst/include \
        -I" /some/path/with-xin-it" \
        -I"/inst/etc" \
        -I"/inst/etc/cling" \
        -I"/inst/etc/cling/plugins/include" \
        -I"/inst/include" \
        -I"/build/root/include" \
        -I"/build/root/include/" \
		-D__ACLIC__  \
		"/pwd/./Foo.C" \
		"/pwd/Foo_C_ACLiC_linkdef.h"
	Info in <TUnixSystem::Load>: loaded library \
		/inst/lib/libCore.so.6.26.00, status 1
	Info in <ACLiC>: compiling the dictionary and script files
	Info in <ACLiC>: cd "/pwd" ; c++ -fPIC \
		-c \
		-g  \
		-std=c++17 \
		-Wno-implicit-fallthrough \
		-Wno-noexcept-type \
		-pipe \
		-W \
		-Woverloaded-virtual \
		-fsigned-char \
		-pthread  \
        -I/inst/include \
        -I"/pwd/ /some/path/with-xin-it" \
        -I"/inst/etc" \
        -I"/inst/etc/cling" \
        -I"/inst/etc/cling/plugins/include" \
        -I"/inst/include" \
        -I"/build/root/include" \
        -I"/build/root/include/"   \
		-D__ACLIC__ \
		"/pwd/Foo_C_ACLiC_dict.cxx" ; \
		c++ -g  "/pwd/Foo_C_ACLiC_dict.o" \
		-shared   \
		"/lib/x86_64-linux-gnu/libm.so" \
		"/lib/x86_64-linux-gnu/libpthread.so" \
		"/lib/x86_64-linux-gnu/libdl.so" \
        "/lib/x86_64-linux-gnu/libzstd.so" \
        "/lib/x86_64-linux-gnu/libz.so" \
        "/lib/x86_64-linux-gnu/liblz4.so" \
        "/lib/x86_64-linux-gnu/liblzma.so" \
        "/lib/x86_64-linux-gnu/libpcre.so" \
        "/lib/x86_64-linux-gnu/libc.so" \
        "/inst/lib/libRint.so.6.26" \
        "/inst/lib/libCore.so.6.26" \
        "/inst/lib/libRIO.so" \
        "/inst/lib/libThread.so.6.26" \
        "/inst/lib/libCling.so" \
        "/inst/lib/libMathCore.so" \
        "/inst/lib/libImt.so.6.26" \
        "/inst/lib/libMultiProc.so.6.26" \
        "/inst/lib/libNet.so.6.26" \
		-o "/pwd/./Foo_C.so"
	Info in <ACLiC>: loading the shared library
	Info in <TInterpreter::TCling::RegisterModule>: \
		Header parsing on demand is active but this is an \
		Aclic library. Disabling it for this library.
	Info in <TUnixSystem::Load>: loaded library /pwd/Foo_C.so, status 0

Note the line 

        -I"/pwd/ /some/path/with-xin-it" \
		
above. 

## More 

ROOT wrongly puts the current working directory into the path in case
the path starts with a white space - I guess there's code like 

	// dir is path 
	if (path[0] != '/')  // Assumes relative path
	  path = Form("%s/%s",gSystem->GetWorkingDirectory(),path);
	  
	  
One should probably use proper tools for this, and, at the very least,
strip `path` of any leading or trailing white-space. 


